library (ggplot2)

projdir <- "/work/MOD3DEV/mqin/R_scripts"
inpLoc   <- paste(projdir, "/Output/",sep="")
figsloc  <- paste(projdir, "/Output",sep="")
figsPre <- figsPre  <- "plot.grid.0.25NOx"

scenName   <- c( "0.25NOx_e1y10_esolv1", "0.25NOx_e2y10_esolv2", "0.25NOx_e3y10_esolv3", "0.25NOx_e4y10_esolv4")#
emisName <- c("1x","2x","3x","4x")
yieldName <- c("1%","5%","10%","CMAQ")

soaType <-c("SOAPM1_SLVOOA_noBG")
nCase = length(scenName)
nType = length(soaType)

hpwhite      <- rgb(255,255,255,maxColorValue=255)
hpgrey        <- rgb(205,204,204,maxColorValue=255)
hpdarkblue    <- rgb( 58, 83,164,maxColorValue=255)
hpblue        <- rgb( 68,123,190,maxColorValue=255)
hpturq        <- rgb(112,205,221,maxColorValue=255)
hpgreen       <- rgb(154,205,112,maxColorValue=255)
hpyellow      <- rgb(245,235, 21,maxColorValue=255)
hporange      <- rgb(245,127, 32,maxColorValue=255)
hpred         <- rgb(237, 34, 36,maxColorValue=255)
hpdarkred     <- rgb(130, 21, 24,maxColorValue=255)
my.colors <- colorRampPalette(c(hpdarkblue,hpblue,hpwhite,hpred,hpdarkred))

print('Making Plots...')

figName = paste( figsloc,"/",figsPre,"SOA.NMB2.pdf",sep="" )

fig.height = 8
fig.width  = 8
cm2inch    = 0.393701
xlabel = c("Relative to base emission")
ylabel = c("SOA yield")

pdf(file = figName,
             height=fig.height*cm2inch, width=fig.width*cm2inch,
             paper="special", bg="white")


for ( itype in 1:nType)  {
for ( icase in 1:nCase ) {

fileName = paste(inpLoc,"SOA_",scenName[icase],"_",soaType[itype],".csv",sep="")

stats<-read.csv(file=fileName,skip =1,sep=",")
tmp<-stats[2:5,c("nmb","nme")]
tmp$emis <- emisName[icase]
tmp$yield <- yieldName 

if (icase ==1){
data = tmp}else{
data <- rbind(data,tmp)
}
}

data$yield <-factor(data$yield, levels= c("1%","CMAQ","5%","10%"))

print(data)

print(ggplot(data = data, aes(x=emis, y=yield, fill=nmb*100),alpha=0.85) + 
  geom_tile(color="white") + 
  geom_text(aes(emis, yield, label = nmb*100), color = "black", size =4) +
  labs(x = xlabel, y = ylabel) +
  scale_fill_gradientn(colours = my.colors(50),limit=c(-60,60),name="NMB (%)",
  guide = guide_colorbar(barwidth = 8, barheight = 1,frame.colour= "black",
         ticks.colour = "black",label.position = "top",
         title.position = "left", title.vjust = 0.2))+
  theme_bw() +
  theme( axis.text = element_text(size=11,colour="black"),
         legend.position="top",
          panel.background = element_rect(colour = "black"),
          panel.grid.major = element_blank(),
          axis.title.x=element_text(size=12,colour="black",vjust = -3),
          axis.title.y=element_text(size=12,colour="black"),
          axis.text.y=element_text(angle=90,hjust=-0.01))

)
}
  dev.off()

