### This routine plots the source attribution of a species from ISAM output.
### Aug 2015  Ben Murphy

### Revised by Momei Qin to plot diurnal variations of SOA (Background removed) 

##### Set Necessary Libraries #####
library(ncdf4)
require(M3)
library(RCurl)
library(XML)
library(fields)
library(gridExtra)
if ( "package:pracma" %in% search() ) { detach("package:pracma", unload=TRUE) }

source('/work/MOD3DEV/mqin/R_scripts/Examples/svPOA_Paper/lib/func_for_CMAQ_files_with_ncdf_rgdal.r')
source('/work/MOD3DEV/mqin/R_scripts/Examples/svPOA_Paper/lib/get.row.column.r')
source('/work/MOD3DEV/mqin/R_scripts/Examples/svPOA_Paper/lib/slurp.sitecompare.R')
source('/work/MOD3DEV/mqin/R_scripts/Examples/svPOA_Paper/lib/plot.tSeries.R')
source('/work/MOD3DEV/mqin/R_scripts/Examples/svPOA_Paper/lib/process_Stats.R')
source('/work/MOD3DEV/mqin/R_scripts/Examples/svPOA_Paper/lib/murphy_toolKit.R')

##### Set File Locations and Names #####
print('Setting File Structure...')
projdir <- "/work/MOD3DEV/mqin/R_scripts"

figsPre  <- "plot.diurnal.AMS"    #Prefix for all Figures made with this script                             
domainName <- "4CALIF1"
gridName   <- "4CALNEX1"
scenName <- c("0.25NOx_mobile","0.25NOx_e3y5_esolv3","0.25NOx_mobile_esolv0")
scenLabel<- c("Baseline","3x emis + 5% yield","Baseline (no VCP)")


nCase = length(scenName)
inpLoc   <- paste(projdir, "/CMAQ_post/Other",sep="")
obsdir   <- paste(projdir, "/Obs",sep="")
figsloc  <- paste(projdir, "/Output",sep="")


##### Set Flags and Parameters for Determining How Code Will Run #####
print('Set Parameters...')

disptz     <- "PST8PDT"
nlay       <- 1

year   <- c("2010", "2010")
month  <- c("05",   "06")    #05
day    <- c("15",   "15")    #14
hour   <- c("00",   "23")    #23
tzIn   <- "PST8PDT"  #Time zone of user-defined time parameters.
                     #Can use: GMT, EST5EDT, PST8PDT, etc
		 # GMT is easiest for looking at model data for its
		 #   own sake. EST5EDT and so on are better for looking
		 #   at whole days from the measurement perspective.
tmlist = loadTime.hr( year, month, day, hour, tzIn )
dhrvec = tmlist$dhrvec
nhrs   = tmlist$nhrs
daynum = tmlist$daynum

#Define Darkness Vector
localTime = dhrvec
attr( localTime, "tzone" ) = "PST8PDT"
localTime = as.numeric(format( localTime, "%H" ))
ldark = localTime <= 8 | localTime > 20

## Set Up Chemical Species of Interest
names    = c('SOA')
obsNames = c('SVOOA_ob', 'LVOOA_ob')
modNames = c('SOA_PM1')
ylims.hi = c(18)
ylims.lo = c(0)
units    = c('ug m-3')
label    = c('SOA')
specDat = list( names=names, obsNames=obsNames, modNames=modNames,
                ylims.hi=ylims.hi, ylims.lo=ylims.lo,
                units=units, label=label)
nspec = length(names)

## Set Up the Order to Put the Site Names in
siteNames = c("Pasadena")
nSite = length( siteNames )

## Pick Statistics to Put on Plot
cstats = c( "nval","mb","mne","r" )

##### Read in Data #####
print('Reading Data...')

##Read in Ste Compare Observation Data and select for Specific Times and Species
site.data = list()
for ( icase in 1:nCase ) {
  modLoc  <- paste(projdir, "/CMAQ_post/Other",sep="")
  modName <- paste("siteCompare_",scenName[icase],"_2010-05-15_2010-06-16_OM_reactivity.csv",sep="")
  fileName = paste( modLoc,modName,sep="/" )

  site.data[[icase]] = slurp.AQS_Hourly( fileName,specDat,dhrvec )

} #End Case Loop


###### Making Plots ######
print('Making Plots...')

filename = paste( figsloc,"/",figsPre,".",names,"_",scenName[icase],"_SOAPM1_SLVOOA_noBG_IEPOX.pdf",sep="" )
statsfile = paste (figsloc,"/",names,"_",scenName[icase],"_SOAPM1_SLVOOA_noBG_IEPOX.csv",sep="") 

##Set Up Figure Book
fig.height = 5.5
fig.width  = 6.5 
cm2inch    = 0.393701
pdf(file = filename, 
             height=fig.height*cm2inch, width=fig.width*cm2inch, 
             paper="special", bg="white")
Oup = 0.02
Odown = 0.02
Oleft = 0.25
Oright = 0.02
outer_mar = c( Odown, Oleft, Oup, Oright) * cm2inch
Iup = 0.1
Idown = 0.1
Ileft = 0.5
Iright = 0.1
inner_mar = c( Idown, Ileft, Iup, Iright ) * cm2inch

#for ( ispec in 1:length(specDat$names) ) {
for ( isite in 1:nSite ) {
mod_BG = 0.5
obs_BG = 2.1

   print (paste('  Print plot for ', specDat$names, ' at ', siteNames[isite],sep=''))
 
   ##Set Variables for Plotting
    # The idea is to make two arrays, one for model data and one for observations.
    #   mod.pl should be of size - time X case
    #   obs.pl should be of size - time X 1
   mod.pl = array( data = NA, dim = c( nhrs, nCase ) )
   r2.pl = vector( )
   mod_mean = vector()
   obs_mean = vector()

   for ( icase in 1:nCase ) {
     tmp = site.data[[icase]][ site.data[[icase]]$SiteId == siteNames[isite], ]
     tmp = merge( tmp, data.frame(dhrvec), by.x="Time", by.y="dhrvec",all.y=TRUE, sort=FALSE )

         mod.pl[ ,icase ] = df2num( tmp[ , specDat$modNames ] ) - mod_BG
#      }
   }
   obs.pl = df2num( tmp[ tmp$SiteId == siteNames[isite], 
                    specDat$obsNames[1] ] ) + 
            df2num( tmp[ tmp$SiteId == siteNames[isite],
                    specDat$obsNames[2] ] ) - obs_BG
 
   for ( icase in 1:nCase ) {
   r2.pl[icase] = cor(mod.pl[,icase],obs.pl,use = "pairwise.complete.obs")
   mod_mean[icase] = mean (mod.pl[,icase],na.rm=TRUE)
   obs_mean = mean(obs.pl,na.rm=TRUE)
  }
   ##Set Plot Properties
   ylims = c(specDat$ylims.lo[1], specDat$ylims.hi[1]) 
   ylabel = bquote( "SOA" ~ " Concentration "  ~ mu * "g m" ^ "-3" )
   titletext = ''
   lleg = 1
   par(fig = c(0,1,0.,1), omi = outer_mar, mai = c(0.4, 0.35, 0.1, 0.05), cex=0.8 )
   lst = 1  #Print Statistics on Plot

   #Use Custom Plot Functions to Make Plots -
   lobsDate = 1  #Only use observations time points when computing model diurnal averages.
   D1 = plot.diurnBox( obs.pl,mod.pl,dhrvec,disptz,titletext,ylims,ylabel,"",scenLabel,
                       lleg,lobsDate,lstats=0 )
   print(r2.pl)
   print(mod_mean)
   print(obs_mean) 
   print( D1$stats )  

}


##Save the plot as a figure
dev.off()

